
/*  A Bison parser, made from parse.y
    by GNU Bison version 1.28  */

#define YYBISON 1  /* Identify Bison output.  */

#define	NE	257
#define	LE	258
#define	GE	259
#define	NEG	260
#define	L_CELL	261
#define	L_RANGE	262
#define	L_VAR	263
#define	L_CONST	264
#define	L_FN0	265
#define	L_FN1	266
#define	L_FN2	267
#define	L_FN3	268
#define	L_FN4	269
#define	L_FNN	270
#define	L_FN1R	271
#define	L_FN2R	272
#define	L_FN3R	273
#define	L_FN4R	274
#define	L_FNNR	275
#define	L_LE	276
#define	L_NE	277
#define	L_GE	278

#line 1 "parse.y"

/*
 * $Id: parse.y,v 1.10 2000/07/22 06:13:16 danny Exp $
 *
 * Copyright  1990, 1992, 1993, 1999 Free Software Foundation, Inc.
 *
 * This file is part of Oleo, the GNU Spreadsheet.
 * 
 * Oleo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * Oleo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Oleo; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#line 44 "parse.y"

#include "funcdef.h"

#include <ctype.h>

#define obstack_chunk_alloc ck_malloc
#define obstack_chunk_free free
#include "obstack.h"
#include "sysdef.h"

#include "global.h"
#include "errors.h"
#include "node.h"
#include "eval.h"
#include "ref.h"

int yylex ();
void yyerror (char *);
VOIDSTAR parse_hash;
extern VOIDSTAR hash_find();

/* This table contains a list of the infix single-char functions */
unsigned char fnin[] = {
	SUM, DIFF, DIV, PROD, MOD, /* AND, OR, */ POW, EQUAL, IF, CONCAT, 0
};

#define YYSTYPE _y_y_s_t_y_p_e
typedef struct node *YYSTYPE;
YYSTYPE parse_return;
YYSTYPE make_list (YYSTYPE, YYSTYPE);

char *instr;
int parse_error = 0;
extern struct obstack tmp_mem;

#ifndef YYSTYPE
#define YYSTYPE int
#endif
#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		133
#define	YYFLAG		-32768
#define	YYNTBASE	41

#define YYTRANSLATE(x) ((unsigned)(x) <= 278 ? yytranslate[x] : 47)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    19,     2,     2,     2,    16,     5,     2,    38,
    39,    14,    12,    40,    13,     2,    15,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     4,     2,     8,
     6,    10,     3,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,    17,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     7,     9,    11,    18,
    20,    21,    22,    23,    24,    25,    26,    27,    28,    29,
    30,    31,    32,    33,    34,    35,    36,    37
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     2,     4,     6,     8,    12,    17,    24,    33,    44,
    49,    54,    59,    66,    73,    82,    91,   100,   109,   114,
   120,   124,   128,   132,   136,   140,   144,   148,   152,   156,
   160,   164,   168,   172,   175,   178,   181,   185,   189,   192,
   194,   198,   200,   202,   204,   208,   210
};

static const short yyrhs[] = {    42,
     0,     1,     0,    23,     0,    46,     0,    24,    38,    39,
     0,    25,    38,    42,    39,     0,    26,    38,    42,    40,
    42,    39,     0,    27,    38,    42,    40,    42,    40,    42,
    39,     0,    28,    38,    42,    40,    42,    40,    42,    40,
    42,    39,     0,    29,    38,    43,    39,     0,    30,    38,
    21,    39,     0,    30,    38,    22,    39,     0,    31,    38,
    21,    40,    42,    39,     0,    31,    38,    22,    40,    42,
    39,     0,    31,    38,    21,    40,    42,    40,    42,    39,
     0,    31,    38,    22,    40,    42,    40,    42,    39,     0,
    32,    38,    21,    40,    42,    40,    42,    39,     0,    32,
    38,    22,    40,    42,    40,    42,    39,     0,    34,    38,
    45,    39,     0,    42,     3,    42,     4,    42,     0,    42,
     5,    42,     0,    42,     8,    42,     0,    42,     9,    42,
     0,    42,     6,    42,     0,    42,     7,    42,     0,    42,
    10,    42,     0,    42,    11,    42,     0,    42,    12,    42,
     0,    42,    13,    42,     0,    42,    14,    42,     0,    42,
    15,    42,     0,    42,    16,    42,     0,    42,    17,    42,
     0,    12,    42,     0,    13,    42,     0,    19,    42,     0,
    38,    42,    39,     0,    38,    42,     1,     0,    38,     1,
     0,    42,     0,    43,    40,    42,     0,    21,     0,    42,
     0,    44,     0,    45,    40,    44,     0,    20,     0,    22,
     0
};

#endif

#if YYDEBUG != 0
static const short yyrline[] = { 0,
    81,    83,    89,    90,    91,    93,    97,   101,   105,   109,
   113,   116,   120,   124,   130,   137,   145,   149,   154,   158,
   169,   173,   177,   181,   185,   189,   193,   197,   201,   205,
   209,   213,   217,   221,   231,   246,   250,   252,   260,   267,
   269,   273,   274,   277,   279,   283,   285
};
#endif


#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)

static const char * const yytname[] = {   "$","error","$undefined.","'?'","':'",
"'&'","'='","NE","'<'","LE","'>'","GE","'+'","'-'","'*'","'/'","'%'","'^'","NEG",
"'!'","L_CELL","L_RANGE","L_VAR","L_CONST","L_FN0","L_FN1","L_FN2","L_FN3","L_FN4",
"L_FNN","L_FN1R","L_FN2R","L_FN3R","L_FN4R","L_FNNR","L_LE","L_NE","L_GE","'('",
"')'","','","line","exp","exp_list","range_exp","range_exp_list","cell", NULL
};
#endif

static const short yyr1[] = {     0,
    41,    41,    42,    42,    42,    42,    42,    42,    42,    42,
    42,    42,    42,    42,    42,    42,    42,    42,    42,    42,
    42,    42,    42,    42,    42,    42,    42,    42,    42,    42,
    42,    42,    42,    42,    42,    42,    42,    42,    42,    43,
    43,    44,    44,    45,    45,    46,    46
};

static const short yyr2[] = {     0,
     1,     1,     1,     1,     3,     4,     6,     8,    10,     4,
     4,     4,     6,     6,     8,     8,     8,     8,     4,     5,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     2,     2,     2,     3,     3,     2,     1,
     3,     1,     1,     1,     3,     1,     1
};

static const short yydefact[] = {     0,
     2,     0,     0,     0,    46,    47,     3,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     1,     4,
    34,    35,    36,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,    39,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     5,
     0,     0,     0,     0,    40,     0,     0,     0,     0,     0,
     0,     0,    42,    43,    44,     0,    38,    37,     0,    21,
    24,    25,    22,    23,    26,    27,    28,    29,    30,    31,
    32,    33,     6,     0,     0,     0,    10,     0,    11,    12,
     0,     0,     0,     0,    19,     0,     0,     0,     0,     0,
    41,     0,     0,     0,     0,    45,    20,     7,     0,     0,
    13,     0,    14,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     8,     0,    15,    16,    17,    18,     0,     9,
     0,     0,     0
};

static const short yydefgoto[] = {   131,
    64,    56,    65,    66,    20
};

static const short yypact[] = {   106,
-32768,   501,   501,   501,-32768,-32768,-32768,   -37,   -21,    12,
    14,    16,    31,    45,    49,    52,    60,   138,   552,-32768,
     5,-32768,-32768,    61,   501,   501,   501,   501,   501,     2,
     9,    11,   478,-32768,    50,   501,   501,   501,   501,   501,
   501,   501,   501,   501,   501,   501,   501,   501,   501,-32768,
   346,   187,   223,   238,   552,    -5,    62,    63,    59,    66,
    68,    71,-32768,   552,-32768,    56,-32768,-32768,   537,    -1,
   207,   207,   562,   562,   562,   562,     5,     5,    86,    86,
    86,    86,-32768,   501,   501,   501,-32768,   501,-32768,-32768,
   501,   501,   501,   501,-32768,   478,   501,   367,   259,   274,
   552,    65,   172,   295,   310,-32768,   552,-32768,   501,   501,
-32768,   501,-32768,   501,   501,   501,   383,   331,   402,   418,
   437,   453,-32768,   501,-32768,-32768,-32768,-32768,   472,-32768,
   113,   117,-32768
};

static const short yypgoto[] = {-32768,
     0,-32768,    24,-32768,-32768
};


#define	YYLAST		579


static const short yytable[] = {    19,
    24,    21,    22,    23,    38,    39,    40,    41,    42,    43,
    44,    45,    46,    47,    48,    49,    25,    35,    46,    47,
    48,    49,    57,    58,    51,    52,    53,    54,    55,    59,
    60,    61,    62,    87,    88,    69,    70,    71,    72,    73,
    74,    75,    76,    77,    78,    79,    80,    81,    82,    26,
    67,    27,    36,    28,    37,    38,    39,    40,    41,    42,
    43,    44,    45,    46,    47,    48,    49,    36,    29,    37,
    38,    39,    40,    41,    42,    43,    44,    45,    46,    47,
    48,    49,    30,    98,    99,   100,    31,   101,    68,    32,
   102,   103,   104,   105,    95,    96,   107,    33,    91,    50,
    89,    90,    49,   111,   112,    92,     1,    93,   117,   118,
    94,   119,   132,   120,   121,   122,   133,     2,     3,   106,
     0,     0,     0,   129,     4,     5,     0,     6,     7,     8,
     9,    10,    11,    12,    13,    14,    15,    16,    34,    17,
     0,     0,     0,    18,     0,     0,     0,     0,     0,     2,
     3,     0,     0,     0,     0,     0,     4,     5,     0,     6,
     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
     0,    17,     0,     0,    36,    18,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    47,    48,    49,    36,
     0,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,     0,     0,     0,     0,     0,     0,
   113,   114,-32768,-32768,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,     0,    36,    84,    37,    38,    39,
    40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
    36,     0,    37,    38,    39,    40,    41,    42,    43,    44,
    45,    46,    47,    48,    49,     0,     0,     0,     0,     0,
     0,    36,    85,    37,    38,    39,    40,    41,    42,    43,
    44,    45,    46,    47,    48,    49,    36,    86,    37,    38,
    39,    40,    41,    42,    43,    44,    45,    46,    47,    48,
    49,     0,     0,     0,     0,     0,     0,    36,   109,    37,
    38,    39,    40,    41,    42,    43,    44,    45,    46,    47,
    48,    49,    36,   110,    37,    38,    39,    40,    41,    42,
    43,    44,    45,    46,    47,    48,    49,     0,     0,     0,
     0,     0,     0,    36,   115,    37,    38,    39,    40,    41,
    42,    43,    44,    45,    46,    47,    48,    49,    36,   116,
    37,    38,    39,    40,    41,    42,    43,    44,    45,    46,
    47,    48,    49,     0,     0,     0,     0,     0,     0,    36,
   124,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,    83,    36,     0,    37,    38,    39,
    40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
     0,     0,     0,     0,    36,   108,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    47,    48,    49,     0,
    36,   123,    37,    38,    39,    40,    41,    42,    43,    44,
    45,    46,    47,    48,    49,     0,     0,     0,     0,    36,
   125,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,     0,    36,   126,    37,    38,    39,
    40,    41,    42,    43,    44,    45,    46,    47,    48,    49,
     0,     0,     0,     0,    36,   127,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    47,    48,    49,     2,
     3,   128,     0,     0,     0,     0,     4,     5,    63,     6,
     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
   130,    17,     2,     3,     0,    18,     0,     0,     0,     4,
     5,     0,     6,     7,     8,     9,    10,    11,    12,    13,
    14,    15,    16,     0,    17,     0,     0,     0,    18,    36,
    97,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    47,    48,    49,    36,     0,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    47,    48,    49,-32768,
-32768,-32768,-32768,    44,    45,    46,    47,    48,    49
};

static const short yycheck[] = {     0,
    38,     2,     3,     4,     6,     7,     8,     9,    10,    11,
    12,    13,    14,    15,    16,    17,    38,    18,    14,    15,
    16,    17,    21,    22,    25,    26,    27,    28,    29,    21,
    22,    21,    22,    39,    40,    36,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    47,    48,    49,    38,
     1,    38,     3,    38,     5,     6,     7,     8,     9,    10,
    11,    12,    13,    14,    15,    16,    17,     3,    38,     5,
     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
    16,    17,    38,    84,    85,    86,    38,    88,    39,    38,
    91,    92,    93,    94,    39,    40,    97,    38,    40,    39,
    39,    39,    17,    39,    40,    40,     1,    40,   109,   110,
    40,   112,     0,   114,   115,   116,     0,    12,    13,    96,
    -1,    -1,    -1,   124,    19,    20,    -1,    22,    23,    24,
    25,    26,    27,    28,    29,    30,    31,    32,     1,    34,
    -1,    -1,    -1,    38,    -1,    -1,    -1,    -1,    -1,    12,
    13,    -1,    -1,    -1,    -1,    -1,    19,    20,    -1,    22,
    23,    24,    25,    26,    27,    28,    29,    30,    31,    32,
    -1,    34,    -1,    -1,     3,    38,     5,     6,     7,     8,
     9,    10,    11,    12,    13,    14,    15,    16,    17,     3,
    -1,     5,     6,     7,     8,     9,    10,    11,    12,    13,
    14,    15,    16,    17,    -1,    -1,    -1,    -1,    -1,    -1,
    39,    40,     6,     7,     8,     9,    10,    11,    12,    13,
    14,    15,    16,    17,    -1,     3,    40,     5,     6,     7,
     8,     9,    10,    11,    12,    13,    14,    15,    16,    17,
     3,    -1,     5,     6,     7,     8,     9,    10,    11,    12,
    13,    14,    15,    16,    17,    -1,    -1,    -1,    -1,    -1,
    -1,     3,    40,     5,     6,     7,     8,     9,    10,    11,
    12,    13,    14,    15,    16,    17,     3,    40,     5,     6,
     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
    17,    -1,    -1,    -1,    -1,    -1,    -1,     3,    40,     5,
     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
    16,    17,     3,    40,     5,     6,     7,     8,     9,    10,
    11,    12,    13,    14,    15,    16,    17,    -1,    -1,    -1,
    -1,    -1,    -1,     3,    40,     5,     6,     7,     8,     9,
    10,    11,    12,    13,    14,    15,    16,    17,     3,    40,
     5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
    15,    16,    17,    -1,    -1,    -1,    -1,    -1,    -1,     3,
    40,     5,     6,     7,     8,     9,    10,    11,    12,    13,
    14,    15,    16,    17,    39,     3,    -1,     5,     6,     7,
     8,     9,    10,    11,    12,    13,    14,    15,    16,    17,
    -1,    -1,    -1,    -1,     3,    39,     5,     6,     7,     8,
     9,    10,    11,    12,    13,    14,    15,    16,    17,    -1,
     3,    39,     5,     6,     7,     8,     9,    10,    11,    12,
    13,    14,    15,    16,    17,    -1,    -1,    -1,    -1,     3,
    39,     5,     6,     7,     8,     9,    10,    11,    12,    13,
    14,    15,    16,    17,    -1,     3,    39,     5,     6,     7,
     8,     9,    10,    11,    12,    13,    14,    15,    16,    17,
    -1,    -1,    -1,    -1,     3,    39,     5,     6,     7,     8,
     9,    10,    11,    12,    13,    14,    15,    16,    17,    12,
    13,    39,    -1,    -1,    -1,    -1,    19,    20,    21,    22,
    23,    24,    25,    26,    27,    28,    29,    30,    31,    32,
    39,    34,    12,    13,    -1,    38,    -1,    -1,    -1,    19,
    20,    -1,    22,    23,    24,    25,    26,    27,    28,    29,
    30,    31,    32,    -1,    34,    -1,    -1,    -1,    38,     3,
     4,     5,     6,     7,     8,     9,    10,    11,    12,    13,
    14,    15,    16,    17,     3,    -1,     5,     6,     7,     8,
     9,    10,    11,    12,    13,    14,    15,    16,    17,     8,
     9,    10,    11,    12,    13,    14,    15,    16,    17
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
#line 3 "/usr/share/misc/bison.simple"
/* This file comes from bison-1.28.  */

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

#ifndef YYSTACK_USE_ALLOCA
#ifdef alloca
#define YYSTACK_USE_ALLOCA
#else /* alloca not defined */
#ifdef __GNUC__
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
#define YYSTACK_USE_ALLOCA
#include <alloca.h>
#else /* not sparc */
/* We think this test detects Watcom and Microsoft C.  */
/* This used to test MSDOS, but that is a bad idea
   since that symbol is in the user namespace.  */
#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
#if 0 /* No need for malloc.h, which pollutes the namespace;
	 instead, just don't use alloca.  */
#include <malloc.h>
#endif
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
/* I don't know what this was needed for, but it pollutes the namespace.
   So I turned it off.   rms, 2 May 1997.  */
/* #include <malloc.h>  */
 #pragma alloca
#define YYSTACK_USE_ALLOCA
#else /* not MSDOS, or __TURBOC__, or _AIX */
#if 0
#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
		 and on HPUX 10.  Eventually we can turn this on.  */
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#endif /* __hpux */
#endif
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc */
#endif /* not GNU C */
#endif /* alloca not defined */
#endif /* YYSTACK_USE_ALLOCA not defined */

#ifdef YYSTACK_USE_ALLOCA
#define YYSTACK_ALLOC alloca
#else
#define YYSTACK_ALLOC malloc
#endif

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	goto yyacceptlab
#define YYABORT 	goto yyabortlab
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, &yylloc, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval, &yylloc)
#endif
#else /* not YYLSP_NEEDED */
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif /* not YYLSP_NEEDED */
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Define __yy_memcpy.  Note that the size argument
   should be passed with type unsigned int, because that is what the non-GCC
   definitions require.  With GCC, __builtin_memcpy takes an arg
   of type size_t, but it can handle unsigned int.  */

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (to, from, count)
     char *to;
     char *from;
     unsigned int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (char *to, char *from, unsigned int count)
{
  register char *t = to;
  register char *f = from;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

#line 217 "/usr/share/misc/bison.simple"

/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into yyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
#ifdef __cplusplus
#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else /* not __cplusplus */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
#endif /* not __cplusplus */
#else /* not YYPARSE_PARAM */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif /* not YYPARSE_PARAM */

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
#ifdef YYPARSE_PARAM
int yyparse (void *);
#else
int yyparse (void);
#endif
#endif

int
yyparse(YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;
  int yyfree_stacks = 0;

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if yyoverflow is a macro.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yystacksize);
#else
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yystacksize);
#endif

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
	  if (yyfree_stacks)
	    {
	      free (yyss);
	      free (yyvs);
#ifdef YYLSP_NEEDED
	      free (yyls);
#endif
	    }
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
#ifndef YYSTACK_USE_ALLOCA
      yyfree_stacks = 1;
#endif
      yyss = (short *) YYSTACK_ALLOC (yystacksize * sizeof (*yyssp));
      __yy_memcpy ((char *)yyss, (char *)yyss1,
		   size * (unsigned int) sizeof (*yyssp));
      yyvs = (YYSTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yyvsp));
      __yy_memcpy ((char *)yyvs, (char *)yyvs1,
		   size * (unsigned int) sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yylsp));
      __yy_memcpy ((char *)yyls, (char *)yyls1,
		   size * (unsigned int) sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

  goto yybackup;
 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  if (yylen > 0)
    yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 1:
#line 82 "parse.y"
{ parse_return=yyvsp[0]; ;
    break;}
case 2:
#line 83 "parse.y"
{
		if(!parse_error)
			parse_error=PARSE_ERR;
		parse_return=0; ;
    break;}
case 5:
#line 91 "parse.y"
{
		yyval=yyvsp[-2]; ;
    break;}
case 6:
#line 93 "parse.y"
{
		(yyvsp[-3])->n_x.v_subs[0]=yyvsp[-1];
		(yyvsp[-3])->n_x.v_subs[1]=(struct node *)0;
		yyval=yyvsp[-3]; ;
    break;}
case 7:
#line 97 "parse.y"
{
		(yyvsp[-5])->n_x.v_subs[0]=yyvsp[-3];
		(yyvsp[-5])->n_x.v_subs[1]=yyvsp[-1];
		yyval=yyvsp[-5]; ;
    break;}
case 8:
#line 101 "parse.y"
{
		(yyvsp[-7])->n_x.v_subs[0]=make_list(yyvsp[-5],yyvsp[-3]);
 		(yyvsp[-7])->n_x.v_subs[1]=yyvsp[-1];
 		yyval=yyvsp[-7];;
    break;}
case 9:
#line 105 "parse.y"
{
		(yyvsp[-9])->n_x.v_subs[0]=make_list(yyvsp[-7],yyvsp[-5]);
 		(yyvsp[-9])->n_x.v_subs[1]=make_list(yyvsp[-3],yyvsp[-1]);
 		yyval=yyvsp[-9];;
    break;}
case 10:
#line 109 "parse.y"
{
		(yyvsp[-3])->n_x.v_subs[0]=(struct node *)0;
		(yyvsp[-3])->n_x.v_subs[1]=yyvsp[-1];
		yyval=yyvsp[-3]; ;
    break;}
case 11:
#line 113 "parse.y"
{
		yyvsp[-3]->n_x.v_subs[0]=yyvsp[-1];
		yyval=yyvsp[-3]; ;
    break;}
case 12:
#line 116 "parse.y"
{
		yyvsp[-3]->n_x.v_subs[0]=yyvsp[-1];
		yyval=yyvsp[-3]; ;
    break;}
case 13:
#line 120 "parse.y"
{
		yyvsp[-5]->n_x.v_subs[0]=yyvsp[-3];
		yyvsp[-5]->n_x.v_subs[1]=yyvsp[-1];
		yyval=yyvsp[-5]; ;
    break;}
case 14:
#line 124 "parse.y"
{
		yyvsp[-5]->n_x.v_subs[0]=yyvsp[-3];
		yyvsp[-5]->n_x.v_subs[1]=yyvsp[-1];
		yyval=yyvsp[-5]; ;
    break;}
case 15:
#line 130 "parse.y"
{
		if(yyvsp[-7]->comp_value!=F_INDEX)
			parse_error=PARSE_ERR;
		yyvsp[-7]->comp_value=F_INDEX2;
		yyvsp[-7]->n_x.v_subs[0]=make_list(yyvsp[-5],yyvsp[-3]);
		yyvsp[-7]->n_x.v_subs[1]=yyvsp[-1];
		yyval=yyvsp[-7]; ;
    break;}
case 16:
#line 137 "parse.y"
{
		if(yyvsp[-7]->comp_value!=F_INDEX)
			parse_error=PARSE_ERR;
		yyvsp[-7]->comp_value=F_INDEX2;
		yyvsp[-7]->n_x.v_subs[0]=make_list(yyvsp[-5],yyvsp[-3]);
		yyvsp[-7]->n_x.v_subs[1]=yyvsp[-1];
		yyval=yyvsp[-7]; ;
    break;}
case 17:
#line 145 "parse.y"
{
		(yyvsp[-7])->n_x.v_subs[0]=make_list(yyvsp[-5],yyvsp[-3]);
 		(yyvsp[-7])->n_x.v_subs[1]=yyvsp[-1];
 		yyval=yyvsp[-7];;
    break;}
case 18:
#line 149 "parse.y"
{
		(yyvsp[-7])->n_x.v_subs[0]=make_list(yyvsp[-5],yyvsp[-3]);
 		(yyvsp[-7])->n_x.v_subs[1]=yyvsp[-1];
 		yyval=yyvsp[-7];;
    break;}
case 19:
#line 154 "parse.y"
{
		(yyvsp[-3])->n_x.v_subs[0]=(struct node *)0;
		(yyvsp[-3])->n_x.v_subs[1]=yyvsp[-1];
		yyval=yyvsp[-3]; ;
    break;}
case 20:
#line 158 "parse.y"
{
		yyvsp[-3]->comp_value=IF;
		yyvsp[-3]->n_x.v_subs[0]=yyvsp[-1];
		yyvsp[-3]->n_x.v_subs[1]=yyvsp[0];
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-4];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[-2];
		yyval=yyvsp[-3]; ;
    break;}
case 21:
#line 169 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 22:
#line 173 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 23:
#line 177 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 24:
#line 181 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 25:
#line 185 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 26:
#line 189 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 27:
#line 193 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 28:
#line 197 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 29:
#line 201 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 30:
#line 205 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 31:
#line 209 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 32:
#line 213 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 33:
#line 217 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[-2];
		yyvsp[-1]->n_x.v_subs[1]=yyvsp[0];
		yyval = yyvsp[-1]; ;
    break;}
case 34:
#line 221 "parse.y"
{
		if(yyvsp[0]->comp_value==CONST_FLT) {
			yyvsp[0]->n_x.v_float= (yyvsp[0]->n_x.v_float);
			yyval=yyvsp[0];
		} else if(yyvsp[0]->comp_value==CONST_INT) {
			yyvsp[0]->n_x.v_int= (yyvsp[0]->n_x.v_int);
			yyval=yyvsp[0];
		} else {
			yyval = yyvsp[0];
		} ;
    break;}
case 35:
#line 231 "parse.y"
{
		if(yyvsp[0]->comp_value==CONST_FLT) {
			yyvsp[0]->n_x.v_float= -(yyvsp[0]->n_x.v_float);
			/* free($1); */
			yyval=yyvsp[0];
		} else if(yyvsp[0]->comp_value==CONST_INT) {
			yyvsp[0]->n_x.v_int= -(yyvsp[0]->n_x.v_int);
			/* free($1); */
			yyval=yyvsp[0];
		} else {
			yyvsp[-1]->comp_value = NEGATE;
			yyvsp[-1]->n_x.v_subs[0]=yyvsp[0];
			yyvsp[-1]->n_x.v_subs[1]=(struct node *)0;
			yyval = yyvsp[-1];
		} ;
    break;}
case 36:
#line 246 "parse.y"
{
		yyvsp[-1]->n_x.v_subs[0]=yyvsp[0];
		yyvsp[-1]->n_x.v_subs[1]=(struct node *)0;
		yyval = yyvsp[-1]; ;
    break;}
case 37:
#line 251 "parse.y"
{ yyval = yyvsp[-1]; ;
    break;}
case 38:
#line 252 "parse.y"
{
		if(!parse_error)
			parse_error=NO_CLOSE;
		;
    break;}
case 39:
#line 260 "parse.y"
{
		if(!parse_error)
			parse_error=NO_CLOSE;
		;
    break;}
case 40:
#line 268 "parse.y"
{ yyval = make_list(yyvsp[0], 0); ;
    break;}
case 41:
#line 270 "parse.y"
{ yyval = make_list(yyvsp[0], yyvsp[-2]); ;
    break;}
case 44:
#line 278 "parse.y"
{ yyval=make_list(yyvsp[0], 0); ;
    break;}
case 45:
#line 280 "parse.y"
{ yyval=make_list(yyvsp[0],yyvsp[-2]); ;
    break;}
case 46:
#line 284 "parse.y"
{ yyval=yyvsp[0]; ;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */
#line 543 "/usr/share/misc/bison.simple"

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -yyn if nec to avoid negative indexes in yycheck.  */
	  for (x = (yyn < 0 ? -yyn : 0);
	       x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (yyn < 0 ? -yyn : 0);
		       x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

  goto yyerrlab1;
yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;

 yyacceptlab:
  /* YYACCEPT comes here.  */
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
  return 0;

 yyabortlab:
  /* YYABORT comes here.  */
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
  return 1;
}
#line 287 "parse.y"


void
yyerror (char * s)
{
	if(!parse_error)
		parse_error=PARSE_ERR;
}

YYSTYPE
make_list (YYSTYPE car, YYSTYPE cdr)
{
	YYSTYPE ret;

	ret=(YYSTYPE)obstack_alloc(&tmp_mem,sizeof(*ret));
	ret->comp_value = 0;
	ret->n_x.v_subs[0]=car;
	ret->n_x.v_subs[1]=cdr;
	return ret;
}

#define ERROR -1

extern struct node *yylval;

unsigned char parse_cell_or_range (char **,struct rng *);
int str_to_col (char ** str);

int
yylex ()
{
	int ch;
	struct node *new;
	int isflt;
	char *begin;
	char *tmp_str;
	unsigned char byte_value;
	int n;

	/* unsigned char *ptr; */
	int nn;
	struct function *fp;
	int tmp_ch;

#ifdef TEST
	if(!instr)
		return ERROR;
#endif
	while(isspace(*instr))
		instr++;
	ch = *instr++;
	if(ch=='(' || ch==',' || ch==')')
		return ch;

	new=(struct node *)obstack_alloc(&tmp_mem,sizeof(struct node));
	new->add_byte=0;
	new->sub_value=0;
	switch(ch) {
	case 0:
		return 0;

	case '0': case '1': case '2': case '3': case '4': case '5': case '6':
	case '7': case '8': case '9': case '.':
		isflt = (ch=='.');

		begin=instr-1;
		tmp_str=instr;

		while(isdigit(*tmp_str) || (!isflt && *tmp_str=='.' && ++isflt))
			tmp_str++;
		if(*tmp_str=='e' || *tmp_str=='E') {
			isflt=1;
			tmp_str++;
			if(*tmp_str=='-' || *tmp_str=='+')
				tmp_str++;
			while(isdigit(*tmp_str))
				tmp_str++;
		}
		if(isflt) {
			new->n_x.v_float=astof((char **)(&begin));
			byte_value=CONST_FLT;
		} else {
			new->n_x.v_int=astol((char **)(&begin));
			if(begin!=tmp_str) {
				begin=instr-1;
				new->n_x.v_float=astof((char **)(&begin));
				byte_value=CONST_FLT;
			} else
				byte_value=CONST_INT;
		}
		ch=L_CONST;
		instr=begin;
		break;

	case '"':
		begin=instr;
		while(*instr && *instr!='"') {
			if(*instr=='\\' && instr[1])
				instr++;
			instr++;
		}
		if(!*instr) {
			parse_error=NO_QUOTE;
			return ERROR;
		}
		tmp_str=new->n_x.v_string=(char *)ck_malloc(1+instr-begin);
		while(begin!=instr) {
			unsigned char n;

			if(*begin=='\\') {
				begin++;
				if(begin[0]>='0' && begin[0]<='7') {
					if(begin[1]>='0' && begin[1]<='7') {
						if(begin[2]>='0' && begin[2]<='7') {
							n=(begin[2]-'0') + (010 * (begin[1]-'0')) + ( 0100 * (begin[0]-'0'));
							begin+=3;
						} else {
							n=(begin[1]-'0') + (010 * (begin[0]-'0'));
							begin+=2;
						}
					} else {
						n=begin[0]-'0';
						begin++;
					}
				} else
					n= *begin++;
				*tmp_str++= n;
			} else
				*tmp_str++= *begin++;
		}
		*tmp_str='\0';
		instr++;
		byte_value=CONST_STR;
		ch=L_CONST;
		break;

	case '+':	case '-':

	case '*':	case '/':	case '%':	case '&':
	/* case '|': */	case '^':	case '=':

	case '?':
	{
		unsigned char *ptr;

		for(ptr= fnin;*ptr;ptr++)
			if(the_funs[*ptr].fn_str[0]==ch)
				break;
#ifdef TEST
		if(!*ptr)
			panic("Can't find fnin[] entry for '%c'",ch);
#endif
		byte_value= *ptr;
	}
		break;

	case ':':
		byte_value=IF;
		break;

	case '!':
	case '<':
	case '>':
		if(*instr!='=') {
			byte_value = (ch=='<') ? LESS : (ch=='>') ? GREATER : NOT;
			break;
		}
		instr++;
		byte_value = (ch=='<') ? LESSEQ : (ch=='>') ? GREATEQ : NOTEQUAL;
		ch = (ch=='<') ? LE : (ch=='>') ? GE : NE;
		break;

	case '\'':
	case ';':
	case '[':
	case ']':
	case '\\':
	case '`':
	case '{':
	case '}':
	case '~':
	bad_chr:
		parse_error=BAD_CHAR;
		return ERROR;

	case '#':
		begin=instr-1;
		while(*instr && (isalnum(*instr) || *instr=='_'))
			instr++;
		ch= *instr;
		*instr=0;
		if(!stricmp(begin,tname))
			byte_value=F_TRUE;
		else if(!stricmp(begin,fname))
			byte_value=F_FALSE;
		else if(!stricmp(begin,iname) && (begin[4]==0 || !stricmp(begin+4,"inity")))
			byte_value=CONST_INF;
		else if(!stricmp(begin,mname) ||
			!stricmp(begin,"#ninf"))
			byte_value=CONST_NINF;
		else if(!stricmp(begin,nname) ||
			!stricmp(begin,"#nan"))
			byte_value=CONST_NAN;
		else {
			for(n=1;n<=ERR_MAX;n++)
				if(!stricmp(begin,ename[n]))
					break;
			if(n>ERR_MAX)
				n=BAD_CHAR;
			new->n_x.v_int=n;
			byte_value=CONST_ERR;
		}
		*instr=ch;
		ch=L_CONST;
		break;

	default:
		if(!Global->a0 && (ch=='@' || ch=='$'))
		   goto bad_chr;

		if(Global->a0 && ch=='@') {
			begin=instr;
			while(*instr && (isalpha(*instr) || isdigit(*instr) || *instr=='_'))
				instr++;
			n=instr-begin;
		} else {
			begin=instr-1;
			byte_value=parse_cell_or_range(&begin,&(new->n_x.v_rng));
			if(byte_value) {
				if((byte_value& ~0x3)==R_CELL)
					ch=L_CELL;
				else
					ch=L_RANGE;
				instr=begin;
				break;
			}

			while(*instr && (isalpha(*instr) || isdigit(*instr) || *instr=='_'))
				instr++;

			n=instr-begin;
			while(isspace(*instr))
				instr++;

			if(*instr!='(') {
				ch=L_VAR;
				byte_value=VAR;
				new->n_x.v_var=find_or_make_var(begin,n);
				break;
			}
		}
		tmp_ch=begin[n];
		begin[n]='\0';
		fp=hash_find(parse_hash,begin);
		begin[n]=tmp_ch;
		byte_value= ERROR;
		if(!fp) {
			parse_error=BAD_FUNC;
			return ERROR;
		}

		if(fp>=the_funs && fp<=&the_funs[USR1])
			byte_value=fp-the_funs;
		else {
			for(nn=0;nn<n_usr_funs;nn++) {
				if(fp>=&usr_funs[nn][0] && fp<=&usr_funs[nn][usr_n_funs[nn]]) {
					byte_value=USR1+nn;
					new->sub_value=fp-&usr_funs[nn][0];
					break;
				}
			}
#ifdef TEST
			if(nn==n_usr_funs) {
				io_error_msg("Couln't turn fp into a ##");
				parse_error=BAD_FUNC;
				return ERROR;
			}
#endif
		}

		if(fp->fn_argn&X_J)
			ch= byte_value==F_IF ? L_FN3 : L_FN2;
		else if(fp->fn_argt[0]=='R' || fp->fn_argt[0]=='E')
			ch=L_FN1R-1+fp->fn_argn-X_A0;
		else
			ch=L_FN0 + fp->fn_argn-X_A0;

		break;
	}
	/* new->node_type=ch; */
	new->comp_value=byte_value;
	yylval=new;
	return ch;
}

/*
 * The return value of parse_cell_or_range() is actually
 *      0 if it doesn't look like a cell or a range,
 *      RANGE | some stuff (bitwise or) if it's a range,
 *      other bits if it's a cell
 */
/*
 * A cell is described by a string such as A0 (letter sequence followed by a number).
 * A cell range is described by two cells separated by a dot or a colon,
 *	such as A0.C3 or B5:C10.
 *
 * In absolute addressing, each part of the description is prefixed by a $ sign.
 *	Example : $A0.$C3, $A$0, or B$5:C$10.
 */
static unsigned char
a0_parse_cell_or_range (char **ptr, struct rng *retp)
{
	unsigned tmpc,tmpr;
	char *p;
	int abz = ROWREL|COLREL;

	p= *ptr;
	tmpc=0;
	if(*p=='$') {
		abz-=COLREL;
		p++;
	}
	if(!isalpha(*p))
		return 0;
	tmpc=str_to_col(&p);
	if(tmpc<MIN_COL || tmpc>MAX_COL)
		return 0;
	if(*p=='$') {
		abz-=ROWREL;
		p++;
	}
	if(!isdigit(*p))
		return 0;
	for(tmpr=0;isdigit(*p);p++)
		tmpr=tmpr*10 + *p - '0';

	if(tmpr<MIN_ROW || tmpr>MAX_ROW)
		return 0;

	if(*p==':' || *p=='.') {
		unsigned tmpc1,tmpr1;

		abz = ((abz&COLREL) ? LCREL : 0)|((abz&ROWREL) ? LRREL : 0)|HRREL|HCREL;
		p++;
		if(*p=='$') {
			abz-=HCREL;
			p++;
		}
		if(!isalpha(*p))
			return 0;
		tmpc1=str_to_col(&p);
		if(tmpc1<MIN_COL || tmpc1>MAX_COL)
			return 0;
		if(*p=='$') {
			abz-=HRREL;
			p++;
		}
		if(!isdigit(*p))
			return 0;
		for(tmpr1=0;isdigit(*p);p++)
			tmpr1=tmpr1*10 + *p - '0';
		if(tmpr1<MIN_ROW || tmpr1>MAX_ROW)
			return 0;

		if(tmpr<tmpr1) {
			retp->lr=tmpr;
			retp->hr=tmpr1;
		} else {
			retp->lr=tmpr1;
			retp->hr=tmpr;
		}
		if(tmpc<tmpc1) {
			retp->lc=tmpc;
			retp->hc=tmpc1;
		} else {
			retp->lc=tmpc1;
			retp->hc=tmpc;
		}
		*ptr= p;
		return RANGE | abz;
	}
	retp->lr = retp->hr = tmpr;
	retp->lc = retp->hc = tmpc;
	*ptr=p;
	return R_CELL | abz;
}

/*
 * Parse a numeric range such as (without the []) :
 *	[-2]
 *	[-2:-3]
 * Return 0 if it's not a range, 1 if it is.
 * "current" should be the current row or column for relative positions.
 */
static int
noa0_numeric_range(char **ptr, int *r1, int *r2, int current)
{
	int	negative = 0, num = 0;
	char	*p;

	for (p=*ptr; *p && (isdigit(*p) || *p == '-' || *p == '+'); p++)
		if (*p == '-')
			negative = 1;
		else if (*p != '+')	/* A digit */
			num = (num * 10) + *p - '0';	/* FIX ME relies on ASCII */

	if (negative)
		*r1 = current - num;
	else
		*r1 = current + num;

	num = 0; negative = 0;
	if (*p == ':' || *p == '.') {	/* it's a range */
		p++;			/* skip colon or dot */
		for (; *p && (isdigit(*p) || *p == '-' || *p == '+'); p++)
			if (*p == '-')
				negative = 1;
			else if (*p != '+')	/* A digit */
				num = (num * 10) + *p - '0';	/* FIX ME relies on ASCII */

		if (negative)
			*r2 = current - num;
		else
			*r2 = current + num;
		*ptr = p;	/* Advance pointer */
		return 1;	/* range */
	}

	*r2 = *r1;	/* both results are the same */
	*ptr = p;	/* Advance pointer */
	return 0;	/* no range */
}

static void
noa0_number(char **ptr, int *r, int current)
{
	int	negative = 0, num = 0;
	char	*p;

	for (p=*ptr; *p && (isdigit(*p) || *p == '-' || *p == '+'); p++)
		if (*p == '-')
			negative = 1;
		else if (*p != '+')	/* A digit */
			num = (num * 10) + *p - '0';	/* FIX ME relies on ASCII */

	if (negative)
		*r = current - num;
	else
		*r = current + num;

	*ptr = p;
}

static char *
noa0_find_end(char *p)
{
	while (*p && (isdigit(*p) || *p == '+' || *p == '-'))
		p++;
	return p;
}

/*
 * Taking relative as well as absolute coordinates into account,
 * potentially swap the order of the range.
 */
static int
SwapEm(int bits, struct rng *retp)
{
	signed short	retbits, r1, r2, c1, c2, x;

	r1 = retp->lr;
	r2 = retp->hr;
	c1 = retp->lc;
	c2 = retp->hc;

	retbits = bits & ~(LRREL|HRREL|LCREL|HCREL);
	if (r1 > r2) {
		x = retp->lr;
		retp->lr = retp->hr;
		retp->hr = x;
		if (LRREL) retbits |= HRREL;
		if (HRREL) retbits |= LRREL;
	} else {
		retbits |= (bits & (LRREL|HRREL));
	}
	if (c1 > c2) {
		x = retp->lc;
		retp->lc = retp->hc;
		retp->hc = x;
		if (LCREL) retbits |= HCREL;
		if (HCREL) retbits |= LCREL;
	} else {
		retbits |= (bits & (LCREL|HCREL));
	}
	return retbits;
}

/*
 * The new implementation of parse_cell_or_range can read non-A0
 *	in SYLK format as well.
 *
 * A single cell is described by R<number>C<number>.
 * A range is described either by :
 *	(Oleo native)	R<number range>C<number range>
 * or
 *	(SYLK)		<cell>:<cell>
 *
 * In absolute addressing, a number is just that.
 * In relative addressing, it is "["<number>"]".
 *	If the relative offset is 0 then it and the brackets can be omitted,
 *	so "rc3" really is a row-relative address.
 *
 * Examples :
 *	(Oleo)		RC[-2:-3]
 *	(SYLK)		RC[-2]:RC[-3]
 */
static unsigned char
noa0_parse_cell_or_range(char **ptr, struct rng *retp)
{
	int		rrange = 0, crange = 0;		/* Are R or C ranges ? */
							/* Are R or C relative ? */
	int		r1rel = 0, c1rel = 0, r2rel = 0, c2rel = 0;
	char		*p, *next;
	int		r1, r2, c1, c2, ret, forget;

	p = *ptr;
	r1 = r2 = cur_row;
	c1 = c2 = cur_col;

	if (*p != 'r' && *p != 'R')
		return 0;

	/* Grab whatever's after the first R */
	p++;
	switch (*p) {
	case 'c':	/* Relative with no row offset */
	case 'C':
		r1 = r2 = cur_row;
		r1rel = r2rel = 1;
		rrange = 0;
		break;
	case '+': case '-':
	case '0': case '1': case '2': case '3':
	case '4': case '5': case '6': case '7':
	case '8': case '9':
		rrange = noa0_numeric_range(&p, &r1, &r2, 0);
		break;
	case '[':
		p++;
		r1rel = r2rel = 1;
		rrange = noa0_numeric_range(&p, &r1, &r2, cur_row);
		if (*p != ']')
			return 0;	/* Invalid string */
		p++;
		break;
	default:
		return 0;	/* Invalid string */
	}

	/* If this is a bracket then we have a mixed absolute/relative range */
	if (*p == '[') {
		p++;	/* Skip [ */
		r2rel = 1;
		noa0_number(&p, &r2, 0);
		r2 += cur_row;
		p++;	/* Skip ] */
	}

	/* Need to find a C now */
	if (*p != 'c' && *p != 'C')
		return 0;	/* Invalid string */

	/* Read whatever's behind the C */
	p++;
	switch (*p) {
	case '.':	/* SYLK style range */
	case ':':
		c1 = c2 = cur_col;
		c1rel = c2rel = 1;
		break;
	case '+': case '-':
	case '0': case '1': case '2': case '3':
	case '4': case '5': case '6': case '7':
	case '8': case '9':
		next = noa0_find_end(p);
		if (*next == '.' || *next == ':') {
			if (*(next+1) == 'r' || *(next+1) == 'R') {
				/* SYLK range; only read one number here */
				noa0_number(&p, &c1, 0);
				break;
			} else if (*(next+1) == '[') {
				noa0_number(&p, &c1, 0);
				p = next+2;
				noa0_number(&p, &c2, cur_col);
				p++;	/* Skip ] */
				c2rel = 1;

				*ptr = p;
				ret = 0;

				retp->lr = r1;
				retp->hr = r2;
				retp->lc = c1;
				retp->hc = c2;

				if (rrange || crange)
					ret |= RANGE;
				else
					ret |= R_CELL;
				if (c1rel)
					ret |= LCREL;
				if (r1rel)
					ret |= LRREL;
				if (c2rel)
					ret |= HCREL;
				if (r2rel)
					ret |= HRREL;

				return SwapEm(ret, retp);
			}
		}
		crange = noa0_numeric_range(&p, &c1, &c2, 0);
		break;
	case '[':
		p++;
		c1rel = 1;
		crange = noa0_numeric_range(&p, &c1, &c2, cur_col);
		if (*p != ']')
			return 0;	/* Invalid string */
		p++;
		break;
	default:
		c1 = c2 = cur_col;
		c1rel = c2rel = 1;
#if 0
		*ptr = p;
		if (rrange || crange)
			return RANGE;	/* FIX ME */
		else
			return R_CELL;	/* FIX ME */
#endif
	}

	if (*p != ':' && *p != '.') {	/* Would be SYLK ranges */
		*ptr = p;
		ret = 0;

		retp->lr = r1;
		retp->hr = r2;
		retp->lc = c1;
		retp->hc = c2;

		if (rrange || crange)
			ret |= RANGE;
		else
			ret |= R_CELL;
		if (c1rel)
			ret |= LCREL;
		if (r1rel)
			ret |= LRREL;
		if (c2rel)
			ret |= HCREL;
		if (r2rel)
			ret |= HRREL;

		return SwapEm(ret, retp);
	}

	/* So now it's a SYLK range */
	p++;				/* Skip . or : */

	if (*p != 'r' && *p != 'R')	/* Need to find second R */
		return 0;

	/* Grab whatever's after the second R */
	p++;
	switch (*p) {
	case 'c':	/* Relative with no row offset */
	case 'C':
		r2 = cur_row;
		r2rel = 1;
		rrange = (r1 == r2) ? 0 : 1;
		break;
	case '+': case '-':
	case '0': case '1': case '2': case '3':
	case '4': case '5': case '6': case '7':
	case '8': case '9':
		rrange = noa0_numeric_range(&p, &r2, &forget, 0);
		break;
	case '[':
		p++;
		r1rel = r2rel = 1;
		rrange = noa0_numeric_range(&p, &r2, &forget, cur_row);
		if (*p != ']')
			return 0;	/* Invalid string */
		p++;
		break;
	default:
		return 0;	/* Invalid string */
	}

	/* Need to find second C now */
	if (*p != 'c' && *p != 'C')
		return 0;	/* Invalid string */

	/* Read whatever's behind the C */
	p++;
	switch (*p) {
	case '.':	/* SYLK style range */
	case ':':
		c2 = cur_col;
		c1rel = c2rel = 1;
		crange = (c1 == c2) ? 0 : 1;
		break;
	case '+': case '-':
	case '0': case '1': case '2': case '3':
	case '4': case '5': case '6': case '7':
	case '8': case '9':
		crange = noa0_numeric_range(&p, &c2, &forget, 0);
		break;
	case '[':
		p++;
		c2rel = 1;
		crange = noa0_numeric_range(&p, &c2, &forget, cur_col);
		if (*p != ']')
			return 0;	/* Invalid string */
		p++;
		break;
	default:
		c2 = cur_col;
		c1rel = c2rel = 1;
	}

	*ptr = p;
	ret = RANGE;

	retp->lr = r1;
	retp->hr = r2;
	retp->lc = c1;
	retp->hc = c2;

	if (c1rel)
		ret |= LCREL;
	if (c2rel)
		ret |= HCREL;
	if (r1rel)
		ret |= LRREL;
	if (r2rel)
		ret |= HRREL;

	return SwapEm(ret, retp);
}

/*
 * The return value of parse_cell_or_range() is
 *      0			if it doesn't look like a cell or a range,
 *      RANGE | bits		if it's a range,
 *      R_CELL | other bits	if it's a cell
 *
 * These macros are defined in eval.h, their meaning is :
 *	R_CELL	(12)		this is a single cell
 *		ROWREL	(1)	the row is relative
 *		COLREL	(2)	the column is relative
 *	RANGE	(16)		this is a range
 *		LRREL	(1)	low row is relative
 *		HRREL	(2)	high row is relative
 *		LCREL	(4)	low column is relative
 *		HCREL	(8)	high column is relative
 */
unsigned char
parse_cell_or_range(char **ptr, struct rng *retp)
{
	char		*p = *ptr;	/* Only for the printing */
	unsigned char	r;

	if (Global->a0)
		r = a0_parse_cell_or_range(ptr, retp);
	else
		r = noa0_parse_cell_or_range(ptr, retp);
#if 0
	/*
	 * Use this to print out whatever parse_cell_or_range() does.
	 */
	fprintf(stderr, "parse_cell_or_range(%s) -> [%d..%d, %d..%d], %d\n",
		p, retp->lr, retp->hr, retp->lc, retp->hc, r);
	fprintf(stderr, "parse_cell_or_range -> remaining string is '%s'\n", *ptr);
#endif
	return r;
}

int
str_to_col (char **str)
{
	int ret;
	char c,cc,ccc;
#if MAX_COL>702
	char cccc;
#endif

	ret=0;
	c=str[0][0];
	if(!isalpha((cc=str[0][1]))) {
		(*str)++;
		return MIN_COL + (isupper(c) ? c-'A' : c-'a');
	}
	if(!isalpha((ccc=str[0][2]))) {
		(*str)+=2;
		return MIN_COL+26 + (isupper(c) ? c-'A' : c-'a')*26 + (isupper(cc) ? cc-'A' : cc-'a');
	}
#if MAX_COL>702
	if(!isalpha((cccc=str[0][3]))) {
		(*str)+=3;
		return MIN_COL+702 + (isupper(c) ? c-'A' : c-'a')*26*26 + (isupper(cc) ? cc-'A' : cc-'a')*26 + (isupper(ccc) ? ccc-'A' : ccc-'a');
	}
	if(!isalpha(str[0][4])) {
		(*str)+=4;
		return MIN_COL+18278 + (isupper(c) ? c-'A' : c-'a')*26*26*26 + (isupper(cc) ? cc-'A' : cc-'a')*26*26 + (isupper(ccc) ? ccc-'A' : ccc-'a')*26 + (isupper(cccc) ? cccc-'A' : cccc-'a');
	}
#endif
	return 0;
}
